@@ -1,4 +1,4 @@
-/* $Id: global.c 4237 2008-03-19 02:32:48Z astyanax $ 
+/* $Id: global.c 4248 2008-03-31 06:25:14Z astyanax $ 
 */
 /**************************************************************************
  *   global.c                                                             *
@@ -501,7 +501,9 @@ void shortcut_init(bool unjustify)
 	;
     const char *nano_writeout_msg =
 	N_("Write the current file to disk");
+#ifndef DISABLE_JUSTIFY
     const char *nano_justify_msg = N_("Justify the current paragraph");
+#endif
     const char *nano_insert_msg =
 	N_("Insert another file into the current one");
     const char *nano_whereis_msg =
@@ -658,7 +660,11 @@ void shortcut_init(bool unjustify)
 #endif
 	exit_msg, IFSCHELP(nano_exit_msg), FALSE, VIEW);
 
-    add_to_funcs(do_exit, MBROWSER|MHELP, exit_msg, IFSCHELP(nano_exit_msg), FALSE, VIEW);
+    add_to_funcs(do_exit, MHELP, exit_msg, IFSCHELP(nano_exit_msg), FALSE, VIEW);
+
+#ifndef DISABLE_BROWSER
+    add_to_funcs(do_exit, MBROWSER, exit_msg, IFSCHELP(nano_exitbrowser_msg), FALSE, VIEW);
+#endif
 
     /* TRANSLATORS: Try to keep this at most 10 characters. */
     add_to_funcs(do_writeout_void, MMAIN, N_("WriteOut"),
@@ -824,9 +830,9 @@ void shortcut_init(bool unjustify)
 
 #ifdef ENABLE_MULTIBUFFER
     add_to_funcs(switch_to_prev_buffer_void, MMAIN, _("Previous File"),
-	nano_prevfile_msg, FALSE, VIEW);
+	IFSCHELP(nano_prevfile_msg), FALSE, VIEW);
     add_to_funcs(switch_to_next_buffer_void, MMAIN, N_("Next File"),
-	nano_nextfile_msg, TRUE, VIEW);
+	IFSCHELP(nano_nextfile_msg), TRUE, VIEW);
 #endif
 
     add_to_funcs(do_verbatim_input, MMAIN, N_("Verbatim Input"),
@@ -1101,7 +1107,7 @@ void shortcut_init(bool unjustify)
     add_to_sclist(MBROWSER|MWHEREISFILE, "M-/", (void *) last_file_msg, 0, TRUE);
     add_to_sclist(MBROWSER|MWHEREISFILE, "M-?", (void *) last_file_msg, 0, TRUE);
 #ifndef NANO_TINY
-    add_to_sclist(MMAIN, "M-[", do_find_bracket, 0, TRUE);
+    add_to_sclist(MMAIN, "M-]", do_find_bracket, 0, TRUE);
     add_to_sclist(MMAIN, "M--", do_scroll_up, 0, TRUE);
     add_to_sclist(MMAIN, "M-_", do_scroll_up, 0, TRUE);
     add_to_sclist(MMAIN, "M-+", do_scroll_down, 0, TRUE);
