@@ -1,4 +1,4 @@
-/* $Id: nano.c 4225 2008-03-13 17:48:16Z astyanax $ */
+/* $Id: nano.c 4247 2008-03-20 05:56:46Z astyanax $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -1250,7 +1250,7 @@ void do_toggle(int flag)
 	)
 	enabled = !enabled;
 
-    desc = flagtostr(flag);
+    desc = _(flagtostr(flag));
     statusbar("%s %s", desc, enabled ? _("enabled") :
 	_("disabled"));
 }
