@@ -1,4 +1,4 @@
-/* $Id: rcfile.c 4221 2008-03-11 03:03:53Z astyanax $ */
+/* $Id: rcfile.c 4245 2008-03-20 05:41:00Z astyanax $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -478,6 +478,10 @@ void parse_include(char *ptr)
     nanorc = full_option;
     lineno = 0;
 
+#ifdef DEBUG
+    fprintf(stderr, "Parsing file \"%s\"\n", full_option);
+#endif
+
     parse_rcfile(rcstream
 #ifdef ENABLE_COLOR
 	, TRUE
@@ -690,6 +694,32 @@ void parse_colors(char *ptr, bool icase)
 }
 #endif /* ENABLE_COLOR */
 
+/* Check whether the user has unmapped every shortcut for a
+sequence we consider 'vital', like the exit function */
+static void check_vitals_mapped(void)
+{
+    subnfunc *f;
+    int v;
+#define VITALS 5
+    void *vitals[VITALS] = { do_exit, do_exit, (void *) cancel_msg, (void *) cancel_msg, (void *) cancel_msg };
+    int inmenus[VITALS] = { MMAIN, MHELP, MWHEREIS, MREPLACE, MGOTOLINE };
+
+    for  (v = 0; v < VITALS; v++) {
+       for (f = allfuncs; f != NULL; f = f->next) {
+           if (f->scfunc == vitals[v] && f->menus & inmenus[v]) {
+               const sc *s = first_sc_for(inmenus[v], f->scfunc);
+               if (!s) {
+                   rcfile_error(N_("Fatal error: no keys mapped for function \"%s\""),
+                       f->desc);
+                   fprintf(stderr, N_("Exiting.  Please use nano with the -I option if needed to adjust your nanorc settings\n"));
+                   exit(1);
+               }
+           break;
+           }
+       }
+    }
+}
+
 /* Parse the rcfile, once it has been opened successfully at rcstream,
  * and close it afterwards.  If syntax_only is TRUE, only allow the file
  * to contain color syntax commands: syntax, color, and icolor. */
@@ -965,6 +995,10 @@ void do_rcfile(void)
 		_("\"%s\" is a device file"), nanorc);
     }
 
+#ifdef DEBUG
+    fprintf(stderr, "Parsing file \"%s\"\n", nanorc);
+#endif
+
     /* Try to open the system-wide nanorc. */
     rcstream = fopen(nanorc, "rb");
     if (rcstream != NULL)
@@ -1022,30 +1056,4 @@ void do_rcfile(void)
 #endif
 }
 
-/* Check whether the user has unmapped every shortcut for a
-sequence we consider 'vital', like the exit function */
-int check_vitals_mapped(void)
-{
-    subnfunc *f;
-    int v;
-#define VITALS 5
-    void *vitals[VITALS] = { do_exit, do_exit, (void *) cancel_msg, (void *) cancel_msg, (void *) cancel_msg };
-    int inmenus[VITALS] = { MMAIN, MHELP, MWHEREIS, MREPLACE, MGOTOLINE };
-
-    for  (v = 0; v < VITALS; v++) {
-	for (f = allfuncs; f != NULL; f = f->next) {
-	    if (f->scfunc == vitals[v] && f->menus & inmenus[v]) {
-		const sc *s = first_sc_for(inmenus[v], f->scfunc);
-		if (!s) {
-		    rcfile_error(N_("Fatal error: no keys mapped for function \"%s\""),
-			f->desc);
-		    fprintf(stderr, N_("Exiting.  Please use nano with the -I option if needed to adjust your nanorc settings\n"));
-		    exit(1);
-		}
-	    break;
-	    }
-	}
-    }
-}
-
 #endif /* ENABLE_NANORC */
