@@ -1,12 +1,12 @@
-/* $Id: search.c,v 1.193 2006/12/13 00:27:45 dolorous Exp $ */
+/* $Id: search.c 4226 2008-03-14 04:08:51Z astyanax $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
- *   Copyright (C) 2005, 2006 David Lawrence Ramsey                       *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -39,21 +39,20 @@ static bool history_changed = FALSE;
 static bool regexp_compiled = FALSE;
 	/* Have we compiled any regular expressions? */
 
-/* Regular expression helper functions. */
-
-/* Compile the given regular expression.  Return value 0 means the
- * expression was invalid, and we wrote an error message on the status
- * bar.  Return value 1 means success. */
-int regexp_init(const char *regexp)
+/* Compile the regular expression regexp to see if it's valid.  Return
+ * TRUE if it is, or FALSE otherwise. */
+bool regexp_init(const char *regexp)
 {
-    int rc = regcomp(&search_regexp, regexp, REG_EXTENDED
+    int rc;
+
+    assert(!regexp_compiled);
+
+    rc = regcomp(&search_regexp, regexp, REG_EXTENDED
 #ifndef NANO_TINY
 	| (ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE)
 #endif
 	);
 
-    assert(!regexp_compiled);
-
     if (rc != 0) {
 	size_t len = regerror(rc, &search_regexp, NULL, 0);
 	char *str = charalloc(len);
@@ -61,11 +60,13 @@ int regexp_init(const char *regexp)
 	regerror(rc, &search_regexp, str, len);
 	statusbar(_("Bad regex \"%s\": %s"), regexp, str);
 	free(str);
-	return 0;
+
+	return FALSE;
     }
 
     regexp_compiled = TRUE;
-    return 1;
+
+    return TRUE;
 }
 
 /* Decompile the compiled regular expression we used in the last
@@ -85,7 +86,7 @@ void not_found_msg(const char *str)
 {
     char *disp;
     int numchars;
- 
+
     assert(str != NULL);
 
     disp = display_string(str, 0, (COLS / 2) + 1, FALSE);
@@ -135,6 +136,9 @@ int search_init(bool replacing, bool use_answer)
 {
     int i = 0;
     char *buf;
+    sc *s;
+    void *func = NULL;
+    bool meta_key = FALSE, func_key = FALSE;
     static char *backupstring = NULL;
 	/* The search string we'll be using. */
 
@@ -172,7 +176,8 @@ int search_init(bool replacing, bool use_answer)
 #ifndef DISABLE_TABCOMP
 	TRUE,
 #endif
-	replacing ? replace_list : whereis_list, backupstring,
+	replacing ? MREPLACE : MWHEREIS, backupstring,
+	&meta_key, &func_key,
 #ifndef NANO_TINY
 	&search_history,
 #endif
@@ -200,6 +205,8 @@ int search_init(bool replacing, bool use_answer)
 #endif
 	_(" (to replace)") : "", buf);
 
+    fflush(stderr);
+
     /* Release buf now that we don't need it anymore. */
     free(buf);
 
@@ -207,49 +214,53 @@ int search_init(bool replacing, bool use_answer)
     backupstring = NULL;
 
     /* Cancel any search, or just return with no previous search. */
-    if (i == -1 || (i < 0 && last_search[0] == '\0') ||
-	    (!replacing && i == 0 && answer[0] == '\0')) {
+    if (i == -1 || (i < 0 && *last_search == '\0') || (!replacing &&
+	i == 0 && *answer == '\0')) {
 	statusbar(_("Cancelled"));
 	return -1;
     } else {
-	switch (i) {
-	    case -2:		/* It's an empty string. */
-	    case 0:		/* It's a new string. */
+	for  (s = sclist; s != NULL; s = s->next)
+	    if ((s->menu & currmenu) && i == s->seq) {
+	        func = s->scfunc;
+	  	break;
+	    }
+
+	if (i == -2 || i == 0 ) {
 #ifdef HAVE_REGEX_H
 		/* Use last_search if answer is an empty string, or
 		 * answer if it isn't. */
-		if (ISSET(USE_REGEXP) &&
-			regexp_init((i == -2) ? last_search :
-			answer) == 0)
+		if (ISSET(USE_REGEXP) && !regexp_init((i == -2) ?
+			last_search : answer))
 		    return -1;
 #endif
-		break;
+		;
 #ifndef NANO_TINY
-	    case TOGGLE_CASE_KEY:
+	} else if (func == (void *) case_sens_msg) {
 		TOGGLE(CASE_SENSITIVE);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
-	    case TOGGLE_BACKWARDS_KEY:
+	} else if (func == (void *) backwards_msg) {
 		TOGGLE(BACKWARDS_SEARCH);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
 #endif
 #ifdef HAVE_REGEX_H
-	    case NANO_REGEXP_KEY:
+	} else if (func == (void *) regexp_msg) {
 		TOGGLE(USE_REGEXP);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
 #endif
-	    case NANO_TOOTHERSEARCH_KEY:
+	} else if (func == (void *) do_replace || 
+	  func == (void *) no_replace_msg) {
 		backupstring = mallocstrcpy(backupstring, answer);
 		return -2;	/* Call the opposite search function. */
-	    case NANO_TOGOTOLINE_KEY:
+	} else if (func == (void *) go_to_line_msg) {
 		do_gotolinecolumn(openfile->current->lineno,
 			openfile->placewewant + 1, TRUE, TRUE, FALSE,
 			TRUE);
 				/* Put answer up on the statusbar and
 				 * fall through. */
-	    default:
+	} else {
 		return -1;
 	}
     }
@@ -444,7 +455,7 @@ void do_search(void)
 	return;
 
     /* If answer is now "", copy last_search into answer. */
-    if (answer[0] == '\0')
+    if (*answer == '\0')
 	answer = mallocstrcpy(answer, last_search);
     else
 	last_search = mallocstrcpy(last_search, answer);
@@ -511,7 +522,7 @@ void do_research(void)
     if (last_search[0] != '\0') {
 #ifdef HAVE_REGEX_H
 	/* Since answer is "", use last_search! */
-	if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
+	if (ISSET(USE_REGEXP) && !regexp_init(last_search))
 	    return;
 #endif
 
@@ -827,7 +838,8 @@ ssize_t do_replace_loop(
 		openfile->current_x += match_len + length_change - 1;
 
 	    /* Cleanup. */
-	    openfile->totsize += length_change;
+	    openfile->totsize += mbstrlen(copy) -
+		mbstrlen(openfile->current->data);
 	    free(openfile->current->data);
 	    openfile->current->data = copy;
 
@@ -873,6 +885,7 @@ void do_replace(void)
 {
     filestruct *edittop_save, *begin;
     size_t begin_x, pww_save;
+    bool meta_key = FALSE, func_key = FALSE;
     ssize_t numreplaced;
     int i;
 
@@ -914,7 +927,8 @@ void do_replace(void)
 #ifndef DISABLE_TABCOMP
 	TRUE,
 #endif
-	replace_list_2, last_replace,
+	MREPLACE2, last_replace,
+	&meta_key, &func_key,
 #ifndef NANO_TINY
 	&replace_history,
 #endif
@@ -974,15 +988,19 @@ void do_replace(void)
 void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	bool interactive, bool save_pos, bool allow_update)
 {
+    bool meta_key = FALSE, func_key = FALSE;
+    const sc *s;
+
     if (interactive) {
 	char *ans = mallocstrcpy(NULL, answer);
 
-	/* Ask for it. */
+	/* Ask for the line and column. */
 	int i = do_prompt(FALSE,
 #ifndef DISABLE_TABCOMP
 		TRUE,
 #endif
-		gotoline_list, use_answer ? ans : "",
+		MGOTOLINE, use_answer ? ans : "",
+		&meta_key, &func_key,
 #ifndef NANO_TINY
 		NULL,
 #endif
@@ -997,7 +1015,9 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	    return;
 	}
 
-	if (i == NANO_TOOTHERWHEREIS_KEY) {
+
+	s = get_shortcut(currmenu, &i, &meta_key, &func_key);
+	if (s && s->scfunc == (void *) gototext_msg) {
 	    /* Keep answer up on the statusbar. */
 	    search_init(TRUE, TRUE);
 
@@ -1011,7 +1031,7 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	if (!parse_line_column(answer, &line, &column) || line < 1 ||
 		column < 1) {
 	    if (i == 0)
-		statusbar(_("Come on, be reasonable"));
+		statusbar(_("Invalid line or column number"));
 	    display_main_list();
 	    return;
 	}
@@ -1310,7 +1330,7 @@ void update_history(filestruct **h, const char *s)
     assert(hage != NULL && hbot != NULL);
 
     /* If this string is already in the history, delete it. */
-    p = find_history(*hage, *hbot, s, strlen(s));
+    p = find_history(*hage, *hbot, s, (size_t)-1);
 
     if (p != NULL) {
 	filestruct *foo, *bar;
