@@ -1,4 +1,4 @@
-/* $Id: text.c 4273 2008-07-12 02:03:35Z astyanax $ */
+/* $Id: text.c 4226 2008-03-14 04:08:51Z astyanax $ */
 /**************************************************************************
  *   text.c                                                               *
  *                                                                        *
@@ -761,7 +761,7 @@ bool do_wrap(filestruct *line)
  * blank, as does a '\n' if newline is TRUE. */
 ssize_t break_line(const char *line, ssize_t goal
 #ifndef DISABLE_HELP
-	, bool newln
+	, bool newline
 #endif
 	)
 {
@@ -781,13 +781,13 @@ ssize_t break_line(const char *line, ssize_t goal
 
 	if (is_blank_mbchar(line)
 #ifndef DISABLE_HELP
-		|| (newln && *line == '\n')
+		|| (newline && *line == '\n')
 #endif
 		) {
 	    blank_loc = cur_loc;
 
 #ifndef DISABLE_HELP
-	    if (newln && *line == '\n')
+	    if (newline && *line == '\n')
 		break;
 #endif
 	}
@@ -800,15 +800,6 @@ ssize_t break_line(const char *line, ssize_t goal
 	/* In fact, the whole line displays shorter than goal. */
 	return cur_loc;
 
-#ifndef DISABLE_HELP
-    if (newln && blank_loc <= 0) {
-       /* If blank was not found or was found only first character,
-        * force line break. */
-       cur_loc -= line_len;
-       return cur_loc;
-    }
-#endif
-
     if (blank_loc == -1) {
 	/* No blank was found that was short enough. */
 	bool found_blank = FALSE;
@@ -819,7 +810,7 @@ ssize_t break_line(const char *line, ssize_t goal
 
 	    if (is_blank_mbchar(line)
 #ifndef DISABLE_HELP
-		|| (newln && *line == '\n')
+		|| (newline && *line == '\n')
 #endif
 		) {
 		if (!found_blank)
@@ -843,11 +834,11 @@ ssize_t break_line(const char *line, ssize_t goal
 
     while (*line != '\0' && (is_blank_mbchar(line)
 #ifndef DISABLE_HELP
-	|| (newln && *line == '\n')
+	|| (newline && *line == '\n')
 #endif
 	)) {
 #ifndef DISABLE_HELP
-	if (newln && *line == '\n')
+	if (newline && *line == '\n')
 	    break;
 #endif
 
@@ -1375,6 +1366,7 @@ void do_justify(bool full_justify)
 
     int kbinput;
     bool meta_key, func_key, s_or_t, ran_func, finished;
+    const sc *s;
 
     /* Move to the beginning of the current line, so that justifying at
      * the end of the last line of the file, if that line isn't blank,
@@ -1690,8 +1682,9 @@ void do_justify(bool full_justify)
      * the keystroke and return. */
     kbinput = do_input(&meta_key, &func_key, &s_or_t, &ran_func,
 	&finished, FALSE);
+    s = get_shortcut(currmenu, &kbinput, &meta_key, &func_key);
 
-    if (s_or_t && kbinput == NANO_UNJUSTIFY_KEY) {
+    if (s && s->scfunc == do_uncut_text) {
 	/* Splice the justify buffer back into the file, but only if we
 	 * actually justified something. */
 	if (first_par_line != NULL) {
@@ -1777,6 +1770,7 @@ bool do_int_spell_fix(const char *word)
     char *save_search, *save_replace;
     size_t match_len, current_x_save = openfile->current_x;
     size_t pww_save = openfile->placewewant;
+    bool meta_key = FALSE, func_key = FALSE;
     filestruct *edittop_save = openfile->edittop;
     filestruct *current_save = openfile->current;
 	/* Save where we are. */
@@ -1863,7 +1857,8 @@ bool do_int_spell_fix(const char *word)
 #ifndef DISABLE_TABCOMP
 		TRUE,
 #endif
-		spell_list, word,
+		MSPELL, word,
+		&meta_key, &func_key,
 #ifndef NANO_TINY
 		NULL,
 #endif
@@ -2346,7 +2341,7 @@ void do_spell(void)
     unlink(temp);
     free(temp);
 
-    currshortcut = main_list;
+    currmenu = MMAIN;
 
     /* If the spell-checker printed any error messages onscreen, make
      * sure that they're cleared off. */
@@ -2370,7 +2365,7 @@ void do_spell(void)
 void do_wordlinechar_count(void)
 {
     size_t words = 0, chars = 0;
-    ssize_t nlines = 0;
+    ssize_t lines = 0;
     size_t current_x_save = openfile->current_x;
     size_t pww_save = openfile->placewewant;
     filestruct *current_save = openfile->current;
@@ -2405,7 +2400,7 @@ void do_wordlinechar_count(void)
     /* Get the total line and character counts, as "wc -l"  and "wc -c"
      * do, but get the latter in multibyte characters. */
     if (old_mark_set) {
-	nlines = openfile->filebot->lineno -
+	lines = openfile->filebot->lineno -
 		openfile->fileage->lineno + 1;
 	chars = get_totsize(openfile->fileage, openfile->filebot);
 
@@ -2414,7 +2409,7 @@ void do_wordlinechar_count(void)
 	unpartition_filestruct(&filepart);
 	openfile->mark_set = TRUE;
     } else {
-	nlines = openfile->filebot->lineno;
+	lines = openfile->filebot->lineno;
 	chars = openfile->totsize;
     }
 
@@ -2426,7 +2421,7 @@ void do_wordlinechar_count(void)
     /* Display the total word, line, and character counts on the
      * statusbar. */
     statusbar(_("%sWords: %lu  Lines: %ld  Chars: %lu"), old_mark_set ?
-	_("In Selection:  ") : "", (unsigned long)words, (long)nlines,
+	_("In Selection:  ") : "", (unsigned long)words, (long)lines,
 	(unsigned long)chars);
 }
 #endif /* !NANO_TINY */
